#!/usr/bin/env python

# $Id: mrcheader.py,v 1.4 2006/10/29 18:40:18 wjiang Exp $
# 
# This program prints out MRC image header information
# by Wen Jiang jiang12@purdue.edu  2006-10-29

from struct import pack,unpack,calcsize
from sys import argv, exit

if len(argv)<2:
	print "Usage: %s <file 1> [<file 2> ... <file n>]" % (argv[0])
	exit(1)

"""
/*******************************************************************************
    MRC image format header definition
*******************************************************************************/
struct mrcH
{
    int         nx;                 /* Number of columns -      */
                                    /* Fastest changing in map. */
    int         ny;

    int         nz;                 /* Number of sections -     */
                                    /* slowest changing in map. */

    int         mode;               /* See modes above. */

    int         nxstart;            /* No. of first column in map, default 0.*/
    int         nystart;            /* No. of first row in map, default 0.*/
    int         nzstart;            /* No. of first section in map,default 0.*/

    int         mx;                 /* Number of intervals along X. */
    int         my;                 /* Number of intervals along Y. */
    int         mz;                 /* Number of intervals along Z. */

    float       xlen;               /* Cell dimensions (Angstroms). */
    float       ylen;               /* Cell dimensions (Angstroms). */
    float       zlen;               /* Cell dimensions (Angstroms). */

    float       alpha;              /* Cell angles (Degrees). */
    float       beta;               /* Cell angles (Degrees). */
    float       gamma;              /* Cell angles (Degrees). */

    int         mapc;               /* Which axis corresponds to Columns.  */
                                    /* (1,2,3 for X,Y,Z.                   */
    int         mapr;               /* Which axis corresponds to Rows.     */
                                    /* (1,2,3 for X,Y,Z.                   */
    int         maps;               /* Which axis corresponds to Sections. */
                                    /* (1,2,3 for X,Y,Z.                   */

    float       amin;               /* Minimum density value. */
    float       amax;               /* Maximum density value. */
    float       amean;              /* Mean density value.    */

    int         ispg;               /* Space group number (0 for images). */

    int         nsymbt;             /* Number of chars used for storing   */
                                    /* symmetry operators. */
    int         user[MRC_USER];     /* MRC_USER = 25 */
	
    float       xorigin;            /* X origin. */
    float       yorigin;            /* Y origin. */
    float       zorigin;            /* Y origin. */

    char        map[4];             /* constant string "MAP "  */
    int         machinestamp;	       /* machine stamp in CCP4 convention: big endian=0x11110000 little endian=0x4444000 */

    float       rms;                /* 	rms deviation of map from mean density */

    int         nlabl;              /* Number of labels being used. */
                                    /* 10 text labels of 80 characters each. */
    char        labels[MRC_NUM_LABELS][MRC_LABEL_SIZE];         /* 10, 80 */
}
"""
MRCN=["nx","ny","nz","mode","nxstart","nystart","nzstart","mx","my","mz","xlen","ylen","zlen",
	"alpha","beta","gamma","mapc","mapr","maps","amin","amax","amean","ispg","nsymbt","user",
	"xorigin","yorigin","zorigin","map","machinestamp","rms","nlabel","labels"]
MRCH="iii i iii iii fff fff iii fff ii 100s fff 4s i f i 800s"

# for each file, do:
for f in argv[1:]:
	# read and upack the header
	fp=open(f,"r+")
	s=fp.read(calcsize(MRCH))
	hed=list(unpack(MRCH,s))

	# display the current header
	print "\n%s header:" % (f)
	for i, n in enumerate(MRCN[:-1]): 
		print "%12s = %s" % (n, hed[i])
	for i in range(hed[-2]):
		label = "label %d" % (i)
		print "%12s = %s" % (label, hed[-1][i*80:(i+1)*80])
